@@ -40,7 +40,7 @@ typedef enum {
 	GTK_HTML_COMMAND_INSERT_RULE_PARAM,
 	GTK_HTML_COMMAND_INSERT_IMAGE_PARAM,
 	GTK_HTML_COMMAND_INSERT_TAB_OR_INDENT_MORE,
-	GTK_HTML_COMMAND_INSERT_TAB_OR_INDENT_MORE_OR_NEXT_CELL,
+	GTK_HTML_COMMAND_INSERT_TAB_OR_NEXT_CELL,
 
 	GTK_HTML_COMMAND_MAKE_LINK,
 	GTK_HTML_COMMAND_REMOVE_LINK,
@@ -87,7 +87,7 @@ typedef enum {
 	GTK_HTML_COMMAND_INDENT_INC,
 	GTK_HTML_COMMAND_INDENT_INC_OR_NEXT_CELL,
 	GTK_HTML_COMMAND_INDENT_DEC,
-	GTK_HTML_COMMAND_INDENT_DEC_OR_PREV_CELL,
+	GTK_HTML_COMMAND_PREV_CELL,
 	GTK_HTML_COMMAND_INDENT_PARAGRAPH,
 
 	GTK_HTML_COMMAND_BREAK_AND_FILL_LINE,
