@@ -36,7 +36,7 @@
 
 #include "htmlcursor.h"
 
-#define _HTML_CURSOR_DEBUG
+/* #define _HTML_CURSOR_DEBUG */
 
 #ifdef _HTML_CURSOR_DEBUG
 static void
@@ -205,6 +205,31 @@ forward (HTMLCursor *cursor)
 	return retval;
 }
 
+static gboolean
+forward_in_flow (HTMLCursor *cursor)
+{
+	gboolean retval;
+
+	retval = TRUE;
+	if (cursor->offset != html_object_get_length (cursor->object)) {
+		if (html_object_is_container (cursor->object)) {
+			HTMLObject *obj;
+
+			obj = cursor->object;
+			while ((retval = forward (cursor)) && cursor->object != obj)
+				;
+		} else
+			retval = html_object_cursor_forward (cursor->object, cursor);
+	} else {
+		if (html_object_next_not_slave (cursor->object))
+			retval = forward (cursor);
+		else
+			retval = FALSE;
+	}
+
+	return retval;
+}
+
 gboolean
 html_cursor_forward (HTMLCursor *cursor, HTMLEngine *engine)
 {
@@ -224,8 +249,6 @@ html_cursor_forward (HTMLCursor *cursor, HTMLEngine *engine)
 	return retval;
 }
 
-
-
 static gboolean
 backward (HTMLCursor *cursor)
 {
@@ -247,6 +270,30 @@ backward (HTMLCursor *cursor)
 	return retval;
 }
 
+static gboolean
+backward_in_flow (HTMLCursor *cursor)
+{
+	gboolean retval;
+
+	retval = TRUE;
+	if (cursor->offset  && html_object_is_container (cursor->object)) {
+		HTMLObject *obj;
+
+		obj = cursor->object;
+		while ((retval = backward (cursor)) && cursor->object != obj)
+			;
+	} else {
+		if (cursor->offset > 1 || !cursor->object->prev)
+			retval = html_object_cursor_backward (cursor->object, cursor);
+		else if (cursor->object->prev)
+			retval = backward (cursor);
+		else
+			retval = FALSE;
+	}
+
+	return retval;
+}
+
 gboolean
 html_cursor_backward (HTMLCursor *cursor,
 		      HTMLEngine *engine)
@@ -538,7 +585,7 @@ html_cursor_end_of_line (HTMLCursor *cursor,
 				     &x, &prev_y);
 
 	while (1) {
-		if (! forward (cursor))
+		if (! forward_in_flow (cursor))
 			return TRUE;
 
 		html_object_get_cursor_base (cursor->object, engine->painter, cursor->offset,
@@ -574,7 +621,7 @@ html_cursor_beginning_of_line (HTMLCursor *cursor,
 				     &x, &prev_y);
 
 	while (1) {
-		if (! backward (cursor))
+		if (! backward_in_flow (cursor))
 			return TRUE;
 
 		html_object_get_cursor_base (cursor->object, engine->painter, cursor->offset,
@@ -657,7 +704,7 @@ html_cursor_follows (const HTMLCursor *a,
 }
 
 
-unicode_char_t
+gunichar
 html_cursor_get_current_char (const HTMLCursor *cursor)
 {
 	HTMLObject *next;
@@ -685,7 +732,7 @@ html_cursor_get_current_char (const HTMLCursor *cursor)
 	return html_text_get_char (HTML_TEXT (next), 0);
 }
 
-unicode_char_t
+gunichar
 html_cursor_get_prev_char (const HTMLCursor *cursor)
 {
 	HTMLObject *prev;
@@ -705,13 +752,34 @@ html_cursor_get_prev_char (const HTMLCursor *cursor)
 gboolean
 html_cursor_beginning_of_paragraph (HTMLCursor *cursor, HTMLEngine *engine)
 {
+	HTMLCursor *copy;
+	HTMLObject *flow;
 	gboolean rv = FALSE;
+	gint level, new_level;
+
+	level = html_object_get_parent_level (cursor->object);
+	flow  = cursor->object->parent;
 
 	if (engine->need_spell_check)
 		html_engine_spell_check_range (engine, engine->cursor, engine->cursor);
 
-	while (cursor->offset || html_object_prev_not_slave (cursor->object)) {
-		backward (cursor);
+	while (1) {
+		if (!cursor->offset) {
+			copy = html_cursor_dup (cursor);
+			if (backward (cursor)) {
+				new_level = html_object_get_parent_level (cursor->object);
+				if (new_level < level
+				    || (new_level == level && flow != cursor->object->parent)) {
+					html_cursor_copy (cursor, copy);
+					break;
+				}
+				html_cursor_destroy (copy);
+			} else
+				break;
+		}
+			else
+				if (!backward (cursor))
+					break;
 		rv = TRUE;
 	}
 
@@ -721,14 +789,34 @@ html_cursor_beginning_of_paragraph (HTMLCursor *cursor, HTMLEngine *engine)
 gboolean
 html_cursor_end_of_paragraph (HTMLCursor *cursor, HTMLEngine *engine)
 {
+	HTMLCursor *copy;
+	HTMLObject *flow;
 	gboolean rv = FALSE;
+	gint level, new_level;
+
+	level = html_object_get_parent_level (cursor->object);
+	flow  = cursor->object->parent;
 
 	if (engine->need_spell_check)
 		html_engine_spell_check_range (engine, engine->cursor, engine->cursor);
 
-	while (cursor->offset < html_object_get_length (cursor->object)
-	       || html_object_next_not_slave (cursor->object)) {
-		forward (cursor);
+	while (1) {
+		if (cursor->offset == html_object_get_length (cursor->object)) {
+			copy = html_cursor_dup (cursor);
+			if (forward (cursor)) {
+				new_level = html_object_get_parent_level (cursor->object);
+				if (new_level < level
+				    || (new_level == level && flow != cursor->object->parent)) {
+					html_cursor_copy (cursor, copy);
+					break;
+				}
+				html_cursor_destroy (copy);
+			} else
+				break;
+		}
+			else
+				if (!forward (cursor))
+					break;
 		rv = TRUE;
 	}
 
