@@ -554,6 +554,7 @@ delete_object (HTMLEngine *e, HTMLObject **ret_object, guint *ret_len, HTMLUndoD
 			*ret_len    = len;
 		}
 		delete_setup_undo (e, object, len, dir);
+		gtk_html_editor_event (e->widget, GTK_HTML_EDITOR_EVENT_DELETE, NULL);
 	}
 }
 
@@ -592,7 +593,7 @@ set_cursor_at_end_of_object (HTMLEngine *e, HTMLObject *o, guint len)
 	e->cursor->offset   = html_object_get_length (e->cursor->object);
 }
 
-static void
+static inline void
 insert_object_do (HTMLEngine *e, HTMLObject *obj, guint len, gboolean check, HTMLUndoDirection dir)
 {
 	HTMLObject *cur;
@@ -700,7 +701,7 @@ insert_object (HTMLEngine *e, HTMLObject *obj, guint len, HTMLUndoDirection dir,
 	/* FIXME for tables */
 	if (HTML_IS_TABLE (obj))
 		append_object (e, obj, len, dir);
-	else {
+	else if (len > 0) {
 		insert_object_do (e, obj, len, check, dir);
 		insert_setup_undo (e, len, dir);
 	}
@@ -754,6 +755,16 @@ insert_empty_paragraph (HTMLEngine *e, HTMLUndoDirection dir)
 	remove_empty_and_merge (e, FALSE, left, right, orig);
 	html_cursor_forward (e->cursor, e);
 
+	/* replace empty text in new empty flow by text with current style */
+	if (html_clueflow_is_empty (HTML_CLUEFLOW (e->cursor->object->parent))) {
+		HTMLObject *flow = e->cursor->object->parent;
+
+		html_clue_remove (HTML_CLUE (flow), e->cursor->object);
+		html_object_destroy (e->cursor->object);
+		e->cursor->object = html_engine_new_text_empty (e);
+		html_clue_append (HTML_CLUE (flow), e->cursor->object);
+	}
+
 	insert_setup_undo (e, 1, dir);
 	g_list_free (left);
 	g_list_free (right);
@@ -858,6 +869,7 @@ html_engine_delete_n (HTMLEngine *e, guint len, gboolean forward)
 	if (html_engine_is_selection_active (e))
 		html_engine_delete (e);
 	else {
+		html_engine_block_selection (e);
 		html_engine_set_mark (e);
 		while (len != 0) {
 			if (forward)
@@ -867,6 +879,7 @@ html_engine_delete_n (HTMLEngine *e, guint len, gboolean forward)
 			len --;
 		}
 		html_engine_delete (e);
+		html_engine_unblock_selection (e);
 	}
 }
 
@@ -968,7 +981,7 @@ append_object (HTMLEngine *e, HTMLObject *o, guint len, HTMLUndoDirection dir)
 	} else {
 		HTMLObject *flow;
 
-		flow  = html_clueflow_new (HTML_CLUEFLOW_STYLE_NORMAL, 0);
+		flow  = html_clueflow_new (HTML_CLUEFLOW_STYLE_NORMAL, 0, HTML_LIST_TYPE_UNORDERED, 1);
 		html_clue_append (HTML_CLUE (flow), o);
 
 		html_object_split (e->cursor->object, e, NULL, e->cursor->offset, 2, &left, &right);
