@@ -44,6 +44,7 @@
 
 #include "htmlcolor.h"
 #include "htmlcolorset.h"
+#include "htmldrawqueue.h"
 #include "htmlengine.h"
 #include "htmlengine-save.h"
 #include "htmlenumutils.h"
@@ -218,6 +219,7 @@ copy (HTMLObject *self,
 
 	/* add dest to image_ptr interests */
 	dimg->image_ptr->interests = g_slist_prepend (dimg->image_ptr->interests, dimg);
+	html_image_pointer_ref (dimg->image_ptr);
 }
 
 static void 
@@ -708,7 +710,7 @@ html_image_init (HTMLImage *image,
 		valign = HTML_VALIGN_BOTTOM;
 	image->valign = valign;
 
-	image->image_ptr = html_image_factory_register (imf, image, filename);
+	image->image_ptr = html_image_factory_register (imf, image, filename, FALSE);
 }
 
 HTMLObject *
@@ -755,19 +757,23 @@ html_image_set_spacing (HTMLImage *image, gint hspace, gint vspace)
 		changed = TRUE;
 	}
 
-	if (changed)
+	if (changed) {
+		html_object_change_set (HTML_OBJECT (image), HTML_CHANGE_ALL_CALC);
 		html_engine_schedule_update (image->image_ptr->factory->engine);
+	}
 }
 
 void
 html_image_set_url (HTMLImage *image, const gchar *url)
 {
-	if (url && strcmp (image->image_ptr->url, url)) {
+	if (url) {
 		HTMLImageFactory *imf = image->image_ptr->factory;
 
 		html_object_change_set (HTML_OBJECT (image), HTML_CHANGE_ALL_CALC);
 		html_image_factory_unregister (imf, image->image_ptr, HTML_IMAGE (image));
-		image->image_ptr = html_image_factory_register (imf, image, url);
+		image->image_ptr = html_image_factory_register (imf, image, url, TRUE);
+		html_object_change_set (HTML_OBJECT (image), HTML_CHANGE_ALL_CALC);
+		html_engine_schedule_update (imf->engine);
 	}
 }
 
@@ -785,6 +791,7 @@ html_image_set_border (HTMLImage *image, gint border)
 {
 	if (image->border != border) {
 		image->border = border;
+		html_object_change_set (HTML_OBJECT (image), HTML_CHANGE_ALL_CALC);
 		html_engine_schedule_update (image->image_ptr->factory->engine);
 	}
 }
@@ -837,8 +844,10 @@ html_image_set_size (HTMLImage *image, gint w, gint h, gboolean pw, gboolean ph)
 		changed = TRUE;
 	}
 
-	if (changed)
+	if (changed) {
+		html_object_change_set (HTML_OBJECT (image), HTML_CHANGE_ALL_CALC);
 		html_engine_schedule_update (image->image_ptr->factory->engine);
+	}
 }
 
 char *image_content_types[] = {"image/*", NULL};
@@ -854,6 +863,49 @@ html_image_factory_types (GtkHTMLStream *stream,
 	return image_content_types;
 }
 
+static void
+update_or_redraw (HTMLImagePointer *ip)
+{
+	GSList *list;
+	gboolean update = FALSE;
+
+	for (list = ip->interests; list; list = list->next) {
+		if (list->data == NULL)
+			update = TRUE;
+		else {
+			HTMLImage *image = HTML_IMAGE (list->data);
+			gint pixel_size = html_painter_get_pixel_size (ip->factory->engine->painter);
+			gint w, h;
+
+			w = get_actual_width (image, ip->factory->engine->painter)
+				+ (image->border * 2 + 2 * image->hspace) * pixel_size;
+			h = get_actual_height (image, ip->factory->engine->painter)
+				+ (image->border * 2 + 2 * image->vspace) * pixel_size;
+
+			/* printf ("%dx%d  <-->  %dx%d\n", w, h, HTML_OBJECT (list->data)->width,
+			   HTML_OBJECT (list->data)->ascent + HTML_OBJECT (list->data)->descent); */
+
+			if (w != HTML_OBJECT (list->data)->width
+			    || h != HTML_OBJECT (list->data)->ascent + HTML_OBJECT (list->data)->descent) {
+				html_object_change_set (HTML_OBJECT (list->data), HTML_CHANGE_ALL_CALC);
+				update = TRUE;
+			}
+		}
+	}
+
+	if (!update) {
+		/* printf ("REDRAW\n"); */
+		for (list = ip->interests; list; list = list->next)
+			if (list->data) // && html_object_is_visible (HTML_OBJECT (list->data)))
+				html_draw_queue_add (ip->factory->engine->draw_queue, HTML_OBJECT (list->data));
+		if (ip->interests)
+			html_draw_queue_flush (ip->factory->engine->draw_queue);
+	} else {
+		/* printf ("UPDATE\n"); */
+		html_engine_schedule_update (ip->factory->engine);
+	}
+}
+
 static void
 html_image_factory_end_pixbuf (GtkHTMLStream *stream,
 			       GtkHTMLStreamStatus status,
@@ -861,9 +913,7 @@ html_image_factory_end_pixbuf (GtkHTMLStream *stream,
 {
 	HTMLImagePointer *ip = user_data;
 
-	html_engine_schedule_update (ip->factory->engine);
-	gtk_object_unref (GTK_OBJECT (ip->loader));
-	ip->loader = NULL;
+	update_or_redraw (ip);
 
 	html_image_pointer_unref (ip);
 }
@@ -871,7 +921,7 @@ html_image_factory_end_pixbuf (GtkHTMLStream *stream,
 static void
 html_image_factory_write_pixbuf (GtkHTMLStream *stream,
 				 const gchar *buffer,
-				 guint size,
+				 size_t size,
 				 gpointer user_data)
 {
 	HTMLImagePointer *p = user_data;
@@ -890,21 +940,11 @@ static void
 html_image_factory_area_prepared (GdkPixbufLoader *loader, HTMLImagePointer *ip)
 {
 	if (!ip->animation) {
-		GSList *cur;
-
 		ip->pixbuf = gdk_pixbuf_loader_get_pixbuf (ip->loader);
 		g_assert (ip->pixbuf);
 
-		/* set change flags on images using this image_ptr */
-		cur = ip->interests;
-		while (cur) {
-			if (cur->data)
-				html_object_change_set (HTML_OBJECT (cur->data), HTML_CHANGE_ALL_CALC);
-			cur = cur->next;
-		}
-
 		gdk_pixbuf_ref (ip->pixbuf);
-		html_engine_schedule_update (ip->factory->engine);
+		update_or_redraw (ip);
 	}
 }
 
@@ -1231,7 +1271,7 @@ html_image_pointer_load (HTMLImagePointer *ip)
 }
 
 HTMLImagePointer *
-html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const char *filename)
+html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const char *filename, gboolean reload)
 {
 	HTMLImagePointer *retval;
 
@@ -1262,7 +1302,8 @@ html_image_factory_register (HTMLImageFactory *factory, HTMLImage *i, const char
 			g_hash_table_insert (factory->loaded_images, retval->url, retval);
 			html_image_pointer_load (retval);
 		}
-	}
+	} else if (reload)
+		html_image_pointer_load (retval);
 
 	html_image_pointer_ref (retval);
 
@@ -1293,11 +1334,13 @@ void
 html_image_factory_unregister (HTMLImageFactory *factory, HTMLImagePointer *pointer, HTMLImage *i)
 {
 	pointer->interests = g_slist_remove (pointer->interests, i);
-	if (pointer->refcount <= 1) {
-		g_assert (pointer->interests);
-		g_hash_table_remove (factory->loaded_images, pointer->url);
-	}
+
 	html_image_pointer_unref (pointer);
+	if (pointer->refcount == 1) {
+		g_assert (pointer->interests == NULL);
+		g_hash_table_remove (factory->loaded_images, pointer->url);
+		html_image_pointer_unref (pointer);
+	}
 }
 
 static void
