@@ -23,7 +23,11 @@
 
 #include <config.h>
 #include <string.h> /* strcmp */
+#include <stdlib.h>
+
+#include <gal/unicode/gunicode.h>
 #include "htmlcolorset.h"
+#include "htmlentity.h"
 #include "htmlpainter.h"
 
 
@@ -34,6 +38,9 @@
 /* Our parent class.  */
 static GtkObjectClass *parent_class = NULL;
 
+static gint calc_text_bytes_delta (const gchar *text, gint len, gint line_offset, gint *translated_len, gboolean tabs);
+static gint translate_text_special_chars (const gchar *text, gchar *translated, gint len, gint line_offset, gboolean tabs);
+
 
 /* GtkObject methods.  */
 
@@ -95,6 +102,9 @@ DEFINE_UNIMPLEMENTED (draw_background);
 DEFINE_UNIMPLEMENTED (draw_embedded);
 
 DEFINE_UNIMPLEMENTED (get_pixel_size);
+DEFINE_UNIMPLEMENTED (get_page_width);
+DEFINE_UNIMPLEMENTED (get_page_height);
+
 
 static void
 init (GtkObject *object, HTMLPainterClass *real_klass)
@@ -152,6 +162,9 @@ class_init (GtkObjectClass *object_class)
 
 	class->get_pixel_size = (gpointer) get_pixel_size_unimplemented;
 
+	class->get_page_width  = (gpointer) get_page_width_unimplemented;
+	class->get_page_height = (gpointer) get_page_height_unimplemented;
+
 	parent_class = gtk_type_class (gtk_object_get_type ());
 }
 
@@ -262,8 +275,7 @@ html_painter_set_font_face (HTMLPainter *painter,
 	g_return_if_fail (HTML_IS_PAINTER (painter));
 
 	if (!painter->font_face || !face || strcmp (painter->font_face, face)) {
-		if (painter->font_face)
-			g_free (painter->font_face);
+		g_free (painter->font_face);
 		painter->font_face = g_strdup (face);
 	}
 }
@@ -301,16 +313,26 @@ html_painter_calc_descent (HTMLPainter *painter,
 guint
 html_painter_calc_text_width (HTMLPainter *painter,
 			      const gchar *text,
-			      guint len,
+			      guint len, gint *line_offset,
 			      GtkHTMLFontStyle font_style,
 			      HTMLFontFace *face)
 {
+	guint width;
+	gchar *translated;
+	gint translated_len;
+
 	g_return_val_if_fail (painter != NULL, 0);
 	g_return_val_if_fail (HTML_IS_PAINTER (painter), 0);
 	g_return_val_if_fail (text != NULL, 0);
 	g_return_val_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT, 0);
 
-	return (* HP_CLASS (painter)->calc_text_width) (painter, text, len, font_style, face);
+	translated = alloca (strlen (text) + calc_text_bytes_delta (text, len, *line_offset, &translated_len,
+								    *line_offset != -1) + 1);
+	*line_offset = translate_text_special_chars (text, translated, len, *line_offset, *line_offset != -1);
+
+	width = (* HP_CLASS (painter)->calc_text_width) (painter, translated, translated_len, font_style, face);
+
+	return width;
 }
 
 
@@ -349,15 +371,117 @@ html_painter_draw_rect (HTMLPainter *painter,
 	(* HP_CLASS (painter)->draw_rect) (painter, x, y, width, height);
 }
 
-void
+static gint
+calc_text_bytes_delta (const gchar *text, gint len, gint line_offset, gint *translated_len, gboolean tabs)
+{
+	gunichar uc;
+	const gchar *s;
+	gint delta, skip, current_len;
+
+	current_len = 0;
+	delta = 0;
+	*translated_len = 0;
+	s = text;
+	while (s && (uc = g_utf8_get_char (s)) && current_len < len) {
+		switch (uc) {
+		case ENTITY_NBSP:
+			delta --;
+			(*translated_len) ++;
+			line_offset ++;
+			break;
+		case '\t':
+			if (tabs) {
+				skip = 8 - (line_offset % 8);
+				delta += skip - 1;
+				line_offset += skip;
+				(*translated_len) += skip;
+			} else {
+				(*translated_len) ++;
+				line_offset ++;
+			}
+			break;
+		default:
+			(*translated_len) ++;
+			line_offset ++;
+		}
+		current_len ++;
+		s = g_utf8_next_char (s);
+	}
+
+	return delta;
+}
+
+static inline void
+put_last (const gchar *s, const gchar **ls, gchar **translated)
+{
+	if (*ls)
+		for (; *ls < s; (*ls) ++, (*translated) ++)
+			**translated = **ls;
+}
+
+static gint
+translate_text_special_chars (const gchar *text, gchar *translated, gint len, gint line_offset, gboolean tabs)
+{
+	gunichar uc;
+	const gchar *s, *ls;
+	gint skip, current_len;
+
+	current_len = 0;
+	s = text;
+	ls = NULL;
+	while (s && (uc = g_utf8_get_char (s)) && current_len < len) {
+		put_last (s, &ls, &translated);
+		switch (uc) {
+		case ENTITY_NBSP:
+			*translated = ' ';
+			translated ++;
+			line_offset ++;
+			ls = NULL;
+			break;
+		case '\t':
+			if (tabs) {
+				skip = 8 - (line_offset % 8);
+				line_offset += skip;
+				for (; skip; skip --, translated ++)
+					*translated = ' ';
+			} else {
+				*translated = ' ';
+				translated ++;
+				line_offset ++;
+			}
+			ls = NULL;
+			break;
+		default:
+			ls = s;
+			line_offset ++;
+		}
+		current_len ++;
+		s = g_utf8_next_char (s);
+	}
+	put_last (s, &ls, &translated);
+	*translated = 0;
+
+	return line_offset;
+}
+
+gint
 html_painter_draw_text (HTMLPainter *painter,
 			gint x, gint y,
-			const gchar *text, gint len)
+			const gchar *text, gint len, gint line_offset)
 {
-	g_return_if_fail (painter != NULL);
-	g_return_if_fail (HTML_IS_PAINTER (painter));
+	gchar *translated;
+	gint translated_len;
 
-	(* HP_CLASS (painter)->draw_text) (painter, x, y, text, len);
+	g_return_val_if_fail (painter != NULL, line_offset);
+	g_return_val_if_fail (HTML_IS_PAINTER (painter), line_offset);
+
+	translated = alloca (strlen (text) + calc_text_bytes_delta (text, len, line_offset, &translated_len,
+								    line_offset != -1) + 1);
+	line_offset = translate_text_special_chars (text, translated, len, line_offset, line_offset != -1);
+
+	(* HP_CLASS (painter)->draw_text) (painter, x, y, translated, translated_len);
+
+	return line_offset;
 }
 
 void
@@ -491,9 +615,9 @@ void
 html_painter_draw_spell_error (HTMLPainter *painter,
 			       gint x, gint y,
 			       const gchar *text,
-			       guint off, gint len)
+			       gint len)
 {
-	(* HP_CLASS (painter)->draw_spell_error) (painter, x, y, text, off, len);
+	(* HP_CLASS (painter)->draw_spell_error) (painter, x, y, text, len);
 }
 
 HTMLFont *
@@ -519,3 +643,15 @@ html_painter_get_space_width (HTMLPainter *painter, GtkHTMLFontStyle style, HTML
 {
 	return html_font_manager_get_font (&painter->font_manager, face, style)->space_width;
 }
+
+guint
+html_painter_get_page_width (HTMLPainter *painter, HTMLEngine *e)
+{
+	return 	(* HP_CLASS (painter)->get_page_width) (painter, e);
+}
+
+guint
+html_painter_get_page_height (HTMLPainter *painter, HTMLEngine *e)
+{
+	return 	(* HP_CLASS (painter)->get_page_height) (painter, e);
+}
